/*
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 * File:  "$Id: tcp.c,v 1.1 2009/07/15 11:05:16 webmail Exp $"
 *
 * First Created on 2008/11/11 by jonas_liu@openfind.com.tw
 * Last Modified on "$Date: 2009/07/15 11:05:16 $" by  $Author: webmail $
 *
 * Copyright (c)2009 Openfind Information Technology INC.
 * All rights reserved.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <fcntl.h>

#include "tcp.h"
#include "toutio.h"
#include "version.h"

M2K_VER_ID("$Id: tcp.c,v 1.1 2009/07/15 11:05:16 webmail Exp $");


#ifndef INADDR_NONE
#define INADDR_NONE	0xffffffff /* -1 return */
#endif



int srv_start(int port)
{
   int sockfd;
   struct sockaddr_in srv_addr;
   int optval,optlen;


   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1){
#ifdef __DEBUG
       perror("srv_start_socket()");
#endif
       return -1;
   }

   optval = 1; optlen = sizeof(optval);
   setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (char *)&optval, optlen);
   memset(&srv_addr, 0, sizeof(srv_addr));
   srv_addr.sin_family = AF_INET;
   srv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
   srv_addr.sin_port = htons(port);

   if ( bind(sockfd, (struct sockaddr *)&srv_addr, sizeof(srv_addr)) == -1){
#ifdef __DEBUG
      perror("srv_start_bind()");
#endif
      close(sockfd);
      return -1;
   }

   if ( listen(sockfd, MAX_LISTEN_BACKLOG) == -1 ){
#ifdef __DEBUG
      perror("srv_start_listen()");
#endif
      close(sockfd);
      return -1;
   }
   return sockfd;
}

int linger_on(int fd)
{
   struct linger LingerOff;

   LingerOff.l_onoff = 1;
   LingerOff.l_linger = 1;
   return setsockopt(fd, SOL_SOCKET, SO_LINGER, (void*)&LingerOff
                                              , sizeof(struct linger));
}


int srv_accept(int sfd)
{
   struct sockaddr_in cli_addr;
   int cli_adrlen;
   int cli_sockfd;

   cli_adrlen = sizeof(cli_addr);
   cli_sockfd = accept(sfd, (struct sockaddr *)&cli_addr, &cli_adrlen);
#ifdef __DEBUG
   if ( cli_sockfd == - 1)
      perror("srv_accept()");
#endif
     

   return cli_sockfd;
}

int srv_accept2(int sfd, char *buf, int size)
{
   struct sockaddr_in cli_addr;
   int cli_adrlen;
   int cli_sockfd;
   char *adr = 0;
   int len;

   cli_adrlen = sizeof(cli_addr);
   cli_sockfd = accept(sfd, (struct sockaddr *)&cli_addr, &cli_adrlen);
   if ( cli_sockfd == - 1){
#ifdef __DEBUG
      perror("srv_accept()");
#endif
   }else
      adr = inet_ntoa( (struct in_addr)cli_addr.sin_addr);
   if ( !adr ){
      *buf = '\0';
      return cli_sockfd;
   }

   len = strlen(adr);
   if ( len <  size )
      strncpy(buf, adr, sizeof(buf)-1);
   else{
      strncpy(buf, adr, size-1);
      buf[size-1] = '\0';
   }

   return cli_sockfd;
}

int sock_out(int fd, int t, char *fmt, ...)
{
   char *buf;
   va_list args;
   int len,ret;

   va_start(args, fmt);
   buf=(char *)0;
   /*buf is from malloc(3)*/
   vasprintf(&buf, fmt, args);
   va_end(args);
   /*cannot malloc(3)*/
   if ( buf == (char *)0 ) return 0;
   len = strlen(buf);
   ret = twriten(fd, buf, len, t);
   free(buf);
   return ret;
}

int sock_noblk(int fd)
{
   int val;

   if ( (val = fcntl(fd, F_GETFL, 0)) == -1)
       return -1;
   val |= O_NONBLOCK;
   return fcntl(fd, F_SETFL, val);
}

/*---------------------------------------------------------------------------
  int tcp_conn() -- open TCP connection with remote (char *)host on (int)port
  const char *host: the hostname of remote machine.
  int port: the port number to open connection.(we convert it to network byte
	    order use htons(3).
  Return file descriptor on success, -1 on fail, -2 on dns error.
 */
int tcp_conn(const char *host, int port)
{

   int sockfd;
   struct sockaddr_in srvaddr;
   unsigned long inaddr;
   struct hostent *hp;

   memset(&srvaddr, 0, sizeof(srvaddr));
   srvaddr.sin_family = AF_INET;
   srvaddr.sin_port = htons(port);

   if ( (inaddr=inet_addr(host)) != INADDR_NONE )
      srvaddr.sin_addr.s_addr = inaddr;
   else{
      if ( !(hp = gethostbyname(host)) ) {
         herror("tcp_conn_gethostbyname()");
         return -2;
      }
      memcpy(&srvaddr.sin_addr, hp->h_addr_list[0], hp->h_length);
   }

   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1){
#ifdef __DEBUG
      perror("socket()");
#endif
      return -1;
   } 

   if ( connect(sockfd, (struct sockaddr *) &srvaddr, sizeof(srvaddr)) == -1 ){
#ifdef __DEBUG
      perror("tcp_conn_connect()");
#endif
      close(sockfd);
      return -1;
   } else
      return sockfd;   
}


#undef __TCP_TEST
#ifdef  __TCP_TEST
int main(int argc, char *argv[])
{
   int port;
   int srv_fd, cli_fd;
   char ident[128];

   if ( argc < 2 ){
      fprintf(stderr,"usage: %s [port]\n", argv[0]);
      exit(-1);
   }

   port = atoi(argv[1]);

   if ( (srv_fd = srv_start(port)) <= 0 )
      exit(-1);
   if ( (cli_fd = srv_accept2(srv_fd, ident, 128)) <= 0 )
      exit(-1);

   write(cli_fd, "you reach my server\n", 21);
   close(cli_fd);
   close(srv_fd);
   return 1;
}
int str_echo()
{
   printf("I am here");

}

#endif/*__TCP_TEST*/
